﻿/**
 * @file
 * VuoCompilerGraphvizParser interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This interface description may be modified and distributed under the terms of the GNU Lesser General Public License (LGPL) version 2 or later.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOCOMPILERGRAPHVIZPARSER_H
#define VUOCOMPILERGRAPHVIZPARSER_H

#include "VuoCompilerGeneratorEdge.hh"
#include "VuoCompilerPassiveEdge.hh"
#include "VuoCompilerPublishedPort.hh"
#include "VuoCable.hh"

class VuoCompiler;

struct Agraph_t;
struct Agnode_t;
typedef struct Agraph_t graph_t; ///< Shorthand for @c Agraph_t.


/**
 * Parses nodes and edges from a .vuo composition file.
 */
class VuoCompilerGraphvizParser
{
private:
	VuoCompiler *compiler;
	graph_t *graph;
	map<string, set<string> > dummyInputNamesForNodeClassName;
	map<string, set<string> > dummyOutputNamesForNodeClassName;
	map<string, VuoNodeClass *> dummyNodeClassForName;
	map<string, VuoNodeClass *> nodeClassForName;
	vector<string> unknownNodeClasses;
	map<string, VuoNode *> nodeForName;
	vector<VuoNode *> orderedNodes;
	map<VuoCompilerNode *, map<VuoCompilerNode *, map<VuoCompilerGeneratorPort *, set<VuoCompilerInputEventPort *> > > > generatorCables;  // generatorCables[fromNode][toNode][fromGenerator] = toPorts
	map<VuoCompilerNode *, map<VuoCompilerNode *, set< pair<VuoCompilerOutputEventPort *, VuoCompilerInputEventPort *> > > > passiveCables;  // passiveCables[fromNode][toNode] = (fromPorts, toPorts)
	map<VuoCompilerNode *, vector<VuoCompilerGeneratorEdge *> > generatorInEdgesForNode;
	map<VuoCompilerNode *, vector<VuoCompilerPassiveEdge *> > passiveInEdgesForNode;
	map<VuoCompilerNode *, vector<VuoCompilerPassiveEdge *> > passiveOutEdgesForNode;
	vector<VuoCompilerPublishedPort *> publishedInputPorts;
	vector<VuoCompilerPublishedPort *> publishedOutputPorts;
	VuoNode *publishedInputNode;

	void initialize(FILE *file, VuoCompiler *compiler = NULL);
	void makeDummyPorts(void);
	void makeDummyNodeClasses(void);
	void makeNodeClasses(void);
	void makeNodes(void);
	void makeCables(void);
	void makeEdges(void);
	void makePublishedPorts(void);
	VuoNodeClass * createPublishedInputNodeClass(VuoNodeClass *dummyNodeClass);
	void checkPortClasses(vector<VuoPortClass *> dummy, vector<VuoPortClass *> actual);
	void extractPortNamesAndConstantVals(Agnode_t *n, map<string, string> &portNameToInitializerVal, map<string, bool> &portHasConstantValueDefined);
	void setInputPortConstantVals(VuoCompilerNode *node, map<string, string> &portNameToInitializerVal, map<string, bool> &portHasConstantValueDefined);

public:
	VuoCompilerGraphvizParser(const string &path, VuoCompiler *compiler = NULL);
	VuoCompilerGraphvizParser(FILE *file, VuoCompiler *compiler = NULL);
	map<string, VuoNode *> getNameToNodeMap(void); // @@@ only used in test; make private
	vector<VuoNode *> getNodes(void);
	vector<VuoCompilerGeneratorEdge *> getGeneratorInEdgesForNode(VuoCompilerNode *node);
	vector<VuoCompilerPassiveEdge *> getPassiveInEdgesForNode(VuoCompilerNode *node);
	vector<VuoCompilerPassiveEdge *> getPassiveOutEdgesForNode(VuoCompilerNode *node);
	vector<VuoCompilerGeneratorEdge *> getGeneratorEdges(void);
	vector<string> getUnknownNodeClasses(void);
	set<VuoCable *> getCables(void);
	vector<VuoCompilerPublishedPort *> getPublishedInputPorts(void);
	vector<VuoCompilerPublishedPort *> getPublishedOutputPorts(void);
	VuoNode * getPublishedInputNode(void);
};

#endif
