﻿/**
 * @file
 * VuoCompilerNode interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This interface description may be modified and distributed under the terms of the GNU Lesser General Public License (LGPL) version 2 or later.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOCOMPILERNODE_H
#define VUOCOMPILERNODE_H

#include "VuoBaseDetail.hh"

#include "VuoNode.hh"

#include "VuoCompilerNodeClass.hh"
#include "VuoCompilerPort.hh"
#include "VuoCompilerInputEventPort.hh"
#include "VuoCompilerInstanceData.hh"

/**
 * The compiler detail class for @c VuoNode.
 */
class VuoCompilerNode : public VuoBaseDetail<VuoNode>
{
private:
	VuoCompilerInstanceData *instanceData;
	string nodeInstanceIdentifier;
	string assignIdentifier(Module *module);
	static const string RuntimeSuffix;

public:
	VuoCompilerNode(VuoNode *baseNode);
	void generateAllocation(Module *module);
	void generateEventFunctionCall(Module *module, BasicBlock *block);
	void generateInitFunctionCall(Module *module, BasicBlock *block);
	void generateFiniFunctionCall(Module *module, BasicBlock *block);
	Value * generatePushedCondition(BasicBlock *block);
	void generatePushedReset(BasicBlock *block);
	void generateReleaseOfAllPortValues(Module *module, BasicBlock *block);
	VuoCompilerInstanceData * getInstanceData(void);
	string getIdentifier(void);
};


#endif
