﻿/**
 * @file
 * VuoCompilerNodeClass interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This interface description may be modified and distributed under the terms of the GNU Lesser General Public License (LGPL) version 2 or later.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOCOMPILERNODECLASS_H
#define VUOCOMPILERNODECLASS_H

#include "VuoBaseDetail.hh"
#include "VuoCompilerModule.hh"

#include "VuoNode.hh"
#include "VuoNodeClass.hh"

#include "VuoCompilerGeneratorPortClass.hh"
#include "VuoCompilerInputEventPortClass.hh"
#include "VuoCompilerInstanceDataClass.hh"
#include "VuoCompilerOutputEventPortClass.hh"
#include "VuoCompilerPortClass.hh"

class VuoCompiler;

/**
 * The compiler detail class for @c VuoNodeClass.
 */
class VuoCompilerNodeClass : public VuoBaseDetail<VuoNodeClass>, public VuoCompilerModule  // Order matters: VuoCompilerModule after VuoBaseDetail - http://stackoverflow.com/a/2254306/238387
{
private:
	Function *eventFunction;
	Function *initFunction;
	Function *finiFunction;
	VuoCompilerInstanceDataClass *instanceDataClass;

	VuoCompilerNodeClass(string className, Module * module);
	VuoCompilerNodeClass(VuoCompilerNodeClass *compilerNodeClass);

	static bool isNodeClass(Module *module);
	void parse(void);
	set<string> globalsToRename(void);
	void parseMetadata(void);
	void parseEventFunction(void);
	void parseInitFunction(void);
	void parseFiniFunction(void);
	VuoCompilerPortClass * getInputPortClassWithName(string portName);
	VuoCompilerPortClass * getOutputPortClassWithName(string portName);

	VuoCompilerInputDataClass * parseInputDataParameter(string annotation, Argument *a);
	VuoCompilerOutputDataClass * parseOutputDataParameter(string annotation, Argument *a);
	VuoCompilerInputEventPortClass * parseInputConductorParameter(string annotation, Argument *a, string &dataPortName);
	VuoCompilerOutputEventPortClass * parseOutputConductorParameter(string annotation, Argument *a, string &dataPortName);
	VuoCompilerInputEventPortClass * parseReceptorParameter(string annotation, Argument *a);
	VuoCompilerGeneratorPortClass * parseGeneratorParameter(string annotation, Argument *a);
	VuoCompilerInstanceDataClass * parseInstanceDataParameter(string annotation, Argument *a);
	VuoType * parseTypeParameter(string annotation);
	string parseDefaultParameter(string annotation);

	friend class TestVuoCompilerType;
	friend class TestVuoCompilerNodeClass;

public:
	VuoNode * newNode(string displayName = "", double x = 0, double y = 0);
	static VuoNodeClass * newNodeClass(string nodeClassName, Module * module);

	string getClassIdentifier(void);
	Function * getEventFunction(void);
	Function * getInitFunction(void);
	Function * getFiniFunction(void);
	VuoCompilerInstanceDataClass * getInstanceDataClass(void);
};

#endif
