/**
 * @file
 * VuoNode interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This interface description may be modified and distributed under the terms of the GNU Lesser General Public License (LGPL) version 2 or later.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUONODE_HH
#define VUONODE_HH

#include "VuoBase.hh"
#include "VuoNodeClass.hh"

class VuoCompilerNode;
class VuoRendererNode;
class VuoPort;

/**
 * This class represents an instance of a @c VuoNodeClass.  There may exist 0 to many @c VuoNodes for each @c VuoNodeClass, depending on the graph being processed.
 *
 * @see VuoCompilerNode
 * @see VuoRendererNode
 */
class VuoNode : public VuoBase<VuoCompilerNode,VuoRendererNode>
{
public:
	VuoNode(VuoNodeClass * nodeClass, string displayName, VuoPort * receptorPort, vector<VuoPort *>inputPorts, vector<VuoPort *> outputPorts, double x, double y);

	VuoNodeClass * getNodeClass(void);

	VuoPort * getInputPortWithName(string portName);
	VuoPort * getOutputPortWithName(string portName);

	vector<VuoPort *> getInputPorts(void);
	vector<VuoPort *> getOutputPorts(void);

	VuoPort * getReceptorPort(void);

	string getDisplayName(void);
	void setDisplayName(string displayName);

	string getGraphvizName(void);
	void setGraphvizName(string graphvizName);

	bool isTypecastNode(void);

	double getX(void);
	void setX(double x);
	double getY(void);
	void setY(double y);
	bool getHadStoredCoords();
	void setHadStoredCoords(bool stored);

	void print(void);

private:
	VuoNodeClass *nodeClass;
	vector<VuoPort *> inputPorts;
	vector<VuoPort *> outputPorts;
	VuoPort *receptorPort;
	string displayName;
	string graphvizName;
	double x,y;
	bool hadStoredCoords;
};

#endif // VUONODE_HH
